import { matchCurrentPath, PluggableRoute } from './index'
import { createAction, bindOn, shareChannel } from 'sunny-js/util/Channel'

shareChannel()
const loadingLoadableComponent = 'loadingLoadableComponent'
const on = bindOn(shareChannel())
type LoadingCallback = (route: PluggableRoute) => void

// 发送loadingLoadableComponent事件
export function loading (route) {
  return shareChannel().next(createAction(loadingLoadableComponent, route))
}
// 监听loadingLoadableComponent事件
export function onRouteLoading (cb: LoadingCallback, ...args) {
  return on(loadingLoadableComponent, cb, ...args)
}

const LoadableComponentLoaded = 'LoadableComponentLoaded'
// 发送LoadableComponentLoaded事件
export function load (route: PluggableRoute) {
  return shareChannel().next(createAction(LoadableComponentLoaded, route))
}
// 监听LoadableComponentLoaded事件
export function onRouteLoad (cb: LoadingCallback, ...args) {
  return on(LoadableComponentLoaded, cb, ...args)
}

type EventHookFn = typeof onRouteLoading | typeof onRouteLoad
/**
 * TransitionSwitch
 * onLoadingHook and onLoadHook
 * - 过滤掉嵌套的父级路由
 * - 返回取消订阅的代码逻辑
 */
function wrapWithEventHook (eventHookFn: EventHookFn) {
  return (callback: LoadingCallback, key?: any) => {
    const subscription = eventHookFn(route => {
      if (matchCurrentPath(route)) { // 过滤掉嵌套的父级路由
        console.debug('current path ' + window.location.pathname, 'matchCurrentPath ' + matchCurrentPath(route), route)
        /**
         * fix Cannot update a component from inside the function body of a different component
         * @see https://github.com/facebook/react/issues/18147
         */
        setTimeout(callback)
      }
    }, key || callback)

    // 可以被取消订阅
    return () => subscription.unsubscribe()
  }
}
export const onPageLoading = wrapWithEventHook(onRouteLoading)
export const onPageLoad = wrapWithEventHook(onRouteLoad)
